
create procedure ADM.DROP_PROCEDURE_IF_EXISTS(in PROCEDURE_NAME char(100))
	begin
	declare STMT char(100);
	declare MSG char(20);
	
	declare continue handler for sqlstate '42704' set MSG = 'ok';
	
	set STMT = 'DROP PROCEDURE ' concat PROCEDURE_NAME;
	prepare S1 from STMT;
	execute S1;
	end%

call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.EXECUTE_DYNAMIC_QUERY')%	
	
create procedure ADM.EXECUTE_DYNAMIC_QUERY(in V_QUERY varchar(2048))
	LANGUAGE SQL
	begin
	
	prepare S1 from V_QUERY;
	execute S1;
	end%
	
call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.DROP_TABLE_IF_EXISTS')%	
	
create procedure ADM.DROP_TABLE_IF_EXISTS(in TABLE_NAME char(100))
	begin
	declare STMT char(100);
	declare MSG char(20);
	
	declare continue handler for sqlstate '42704' set MSG = 'ok';
	
	set STMT = 'DROP TABLE ' concat TABLE_NAME;
	prepare S1 from STMT;
	execute S1;
	end%
	
call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.ADD_SHARED_DISKS_NEW_STATUS')%		
	
-- create a new table SHARED_DISK_STATUS_HIST and updates SHARED_DISK_INFO only if needed
CREATE procedure ADM.ADD_SHARED_DISKS_NEW_STATUS () 
	LANGUAGE SQL
	BEGIN
	DECLARE v_count INTEGER DEFAULT 0;
	declare v_stmt varchar(1024);
	
	SELECT count(*) INTO v_count 
	  from sysibm.tables where upper(table_name) = 'SHARED_DISK_STATUS_HIST' and upper(table_schema) = 'ADM';
  
	IF (v_count = 0) THEN

		call ADM.DROP_TABLE_IF_EXISTS('ADM.SHARED_DISK_STATUS_HIST');

		call ADM.EXECUTE_DYNAMIC_QUERY('
			CREATE TABLE ADM.SHARED_DISK_STATUS_HIST
			(
			   HOSTNAME varchar(254) NOT NULL,
			   EXPORTED_DIRECTORY varchar(512) NOT NULL,
			   CUSTOMER_ID bigint NOT NULL,
			   STATUS smallint NOT NULL
			)'
		);

		call ADM.EXECUTE_DYNAMIC_QUERY('
			create unique index ADM.SHARED_HIST_T1 on ADM.SHARED_DISK_STATUS_HIST
			(
				HOSTNAME,
				EXPORTED_DIRECTORY,
				CUSTOMER_ID
			)'
		);

		call ADM.EXECUTE_DYNAMIC_QUERY(
			'grant select, insert, delete, update, alter on ADM.SHARED_DISK_STATUS_HIST to user TLMSRV'
		);
		
		call ADM.EXECUTE_DYNAMIC_QUERY('
			comment on table ADM.SHARED_DISK_STATUS_HIST is 
				''Stores history about shared disk statuses'''
		);
		call ADM.EXECUTE_DYNAMIC_QUERY('
			comment on column ADM.SHARED_DISK_STATUS_HIST.HOSTNAME is 
				''Name of the host exporting the file system'''
		);
		call ADM.EXECUTE_DYNAMIC_QUERY('
			comment on column ADM.SHARED_DISK_STATUS_HIST.EXPORTED_DIRECTORY is 
				''Directory exported from the host'''
		);
		call ADM.EXECUTE_DYNAMIC_QUERY('
			comment on column ADM.SHARED_DISK_STATUS_HIST.CUSTOMER_ID is 
				''Link to adm.customer table'''
		);
		call ADM.EXECUTE_DYNAMIC_QUERY('
			comment on column ADM.SHARED_DISK_STATUS_HIST.STATUS is 
				''Shared disk status (0 - normal, 1 - do not scan)'''
		);		

		call ADM.EXECUTE_DYNAMIC_QUERY(
			'ALTER TABLE ADM.SHARED_DISK_INFO ADD COLUMN STATUS SMALLINT DEFAULT 0'
		);

		call ADM.EXECUTE_DYNAMIC_QUERY(
			'comment on column ADM.SHARED_DISK_INFO.STATUS is 
				''Shared disk status (0 - normal, 1 - do not scan)'''
		);

		call ADM.EXECUTE_DYNAMIC_QUERY('
			create trigger ADM.SHARED_FP2_1
				after insert on ADM.SHARED_DISK_INFO
				referencing new as NEW for each row mode db2sql
				when (
					(select count(*) 
					from ADM.SHARED_DISK_STATUS_HIST h 
					where h.HOSTNAME = NEW.HOSTNAME
					  and h.EXPORTED_DIRECTORY = NEW.EXPORTED_DIRECTORY
					  and h.CUSTOMER_ID = NEW.CUSTOMER_ID) = 0
				)
				insert into ADM.SHARED_DISK_STATUS_HIST (
					HOSTNAME,
					EXPORTED_DIRECTORY,
					CUSTOMER_ID,
					STATUS)
				values (
					NEW.HOSTNAME,
					NEW.EXPORTED_DIRECTORY,
					NEW.CUSTOMER_ID,
					0)'
		);
		
		call ADM.EXECUTE_DYNAMIC_QUERY('
			create trigger ADM.SHARED_FP2_2
				before insert on ADM.SHARED_DISK_INFO
				referencing new as NEW for each row mode db2sql
				when (
					(select count(*) 
					from ADM.SHARED_DISK_STATUS_HIST h 
					where h.HOSTNAME = NEW.HOSTNAME
					  and h.EXPORTED_DIRECTORY = NEW.EXPORTED_DIRECTORY
					  and h.CUSTOMER_ID = NEW.CUSTOMER_ID) = 1
				)
				SET NEW.STATUS =
				  (select h.STATUS from ADM.SHARED_DISK_STATUS_HIST h
					where h.HOSTNAME = NEW.HOSTNAME
					  and h.EXPORTED_DIRECTORY = NEW.EXPORTED_DIRECTORY
					  and h.CUSTOMER_ID = NEW.CUSTOMER_ID)'
		);

		call ADM.EXECUTE_DYNAMIC_QUERY('
			create trigger ADM.SHARED_FP2_3
				after update of STATUS on ADM.SHARED_DISK_INFO
				referencing new as NEW for each row mode db2sql
				update ADM.SHARED_DISK_STATUS_HIST h set h.STATUS = NEW.STATUS
					  where h.HOSTNAME = NEW.HOSTNAME
					  and h.EXPORTED_DIRECTORY = NEW.EXPORTED_DIRECTORY
					  and h.CUSTOMER_ID = NEW.CUSTOMER_ID'
		);

		call Sysproc.admin_cmd(
			'REORG TABLE ADM.SHARED_DISK_INFO'
		);
		
		call Sysproc.admin_cmd(
			'runstats on table ADM.SHARED_DISK_INFO and indexes all'
		);
		
		call Sysproc.admin_cmd(
			'REORG TABLE ADM.SHARED_DISK_STATUS_HIST'
		);
		
		call Sysproc.admin_cmd(
			'runstats on table ADM.SHARED_DISK_STATUS_HIST and indexes all'
		);		
		
		call ADM.EXECUTE_DYNAMIC_QUERY(
			'insert into ADM.SHARED_DISK_STATUS_HIST 
				select HOSTNAME, EXPORTED_DIRECTORY, CUSTOMER_ID, STATUS 
				from adm.SHARED_DISK_INFO'
		);
	
	END IF;
  
	commit;
	END
%

call ADM.ADD_SHARED_DISKS_NEW_STATUS()%

call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.ADD_SHARED_DISKS_NEW_STATUS')%		
call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.DROP_TABLE_IF_EXISTS')%	
call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.EXECUTE_DYNAMIC_QUERY')%	
drop procedure ADM.DROP_PROCEDURE_IF_EXISTS%
